## Fitting
# Univariate Garch-Fit


fitGARCH <- function(Y,T, p =0.025){
  n <- length(Y)-T
  # Schaetze AR(1)-Garch(1,1) Koeffizienten mit t-verteilter bedingter Verteilung auf den Daten bis zum Zeitpunkt T
  bb <- garchFit(~arma(1,0)+garch(1,1),data=Y[1:T], include.mean = FALSE, cond.dist = c("norm"),trace=F, algorithm = c("nlminb"), hessian = c("rcd"))@fit$matcoef[,1]
  #Initial
  Y0 <- 0
  v0 <- 0
  s0 <- sd(Y)
 # Start
  sigma <- sqrt(bb[2] + bb[4]*s0^2)
  v <- Y[1]-bb[1]*Y0
  e <- 0
  H <- 0
  #ES <- -bb[1]*Y[1]-sigma*EST
  #VaR <- -bb[1]*Y[1]-sigma* (qstd(1-p,nu=nu, sd=1))
  #further
  for(i in 2:(T+n)){
  sigma[i] <- sqrt(bb[2] + bb[3]*v[i-1]^2+bb[4]*sigma[i-1]^2)
  e[i] <-   (Y[i] - bb[1] *Y[i-1])/(sigma[i])
  v[i] <- sigma[i]*e[i]
# ES[i] <- -bb[1]*Y[i-1]-sigma[i]*(EST )
# VaR[i] <- -bb[1]*Y[i-1]-sigma[i]*(qnorm(1-p))
  H[i] <- 1/p *(p- pnorm(e[i])) *    (pnorm(e[i])<p)
 }
 return(list(H[-(1:T)],bb))     #H[-(1:T)]
 }

# Fitting m-Garch Prozzes, each univariate
fitmGARCH <- function(Y,Time){
    Y <- as.matrix(Y)
    m <- dim(Y)[2]
    n <- dim(Y)[1]-Time
    H <- matrix(NA,nrow=n,ncol=m)
    BB <-matrix(NA,nrow=4,ncol=m)
    for(k in 1:m) {
      fit <- fitGARCH(Y[,k],Time)
      H[,k] <- fit[[1]]
      BB[,k] <- fit[[2]]
    }
    if(any(BB[4,]<0.1)) return(0)
    return(H)
    }